/* mbed Microcontroller Library
 * Copyright (c) 2006-2013 ARM Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "mbed.h"
#include "BLEDevice.h"
#include "HeartRateService.h"
#include "BatteryService.h"
#include "DeviceInformationService.h"

BLEDevice  ble;
DigitalOut indicatorLed1(LED1);
DigitalOut indicatorLed2(LED2);

#define NEED_CONSOLE_OUTPUT 0 /* Set this if you need debug messages on the console;
                               * it will have an impact on code-size and power consumption. */

#if NEED_CONSOLE_OUTPUT
Serial  pc(USBTX, USBRX);
#define DEBUG(...) { pc.printf(__VA_ARGS__); }
#else
#define DEBUG(...) /* nothing */
#endif /* #if NEED_CONSOLE_OUTPUT */

// Sample interval (uS)
volatile uint32_t sampleIntervalUs = 5000;

// Timer to do the sampling etc
Timer intervalTimer;
Timeout sampleTimeout;

const static char     DEVICE_NAME[]        = "HRMonitor";
static const uint16_t uuid16_list[]        = {GattService::UUID_HEART_RATE_SERVICE,
                                              GattService::UUID_BATTERY_SERVICE,
                                              GattService::UUID_DEVICE_INFORMATION_SERVICE};
static volatile bool  triggerSensorPolling = false;

void disconnectionCallback(Gap::Handle_t handle, Gap::DisconnectionReason_t reason)
{
    DEBUG("Disconnected handle %u!\n\r", handle);
    DEBUG("Restarting the advertising process\n\r");
    ble.startAdvertising();
}

static volatile int callbackCount = 0;
void periodicCallback(void)
{
    callbackCount++;
    if (callbackCount == 200)
    {
        callbackCount = 0;
        indicatorLed1 = !indicatorLed1;
    }       
    
    /* Note that the periodicCallback() executes in interrupt context, so it is safer to do
     * heavy-weight sensor polling from the main thread. */
    triggerSensorPolling = true;
//    sampleTimeout.attach_us(periodicCallback, sampleIntervalUs);
}

int bleUpdateCount = 0;
int main(void)
{
    indicatorLed1 = 1;
    indicatorLed2 = 1;
    
    // Start timer and sampling
    intervalTimer.start();
//    sampleTimeout.attach_us(periodicCallback, sampleIntervalUs);
    
    Ticker ticker;
//    ticker.attach(periodicCallback, 1);
    ticker.attach_us(periodicCallback, sampleIntervalUs);

    DEBUG("Initialising the nRF51822\n\r");
    ble.init();
    ble.onDisconnection(disconnectionCallback);

    /* setup advertising */
    ble.accumulateAdvertisingPayload(GapAdvertisingData::BREDR_NOT_SUPPORTED | GapAdvertisingData::LE_GENERAL_DISCOVERABLE);
    ble.accumulateAdvertisingPayload(GapAdvertisingData::COMPLETE_LIST_16BIT_SERVICE_IDS, (uint8_t *)uuid16_list, sizeof(uuid16_list));
    ble.accumulateAdvertisingPayload(GapAdvertisingData::GENERIC_HEART_RATE_SENSOR);
    ble.accumulateAdvertisingPayload(GapAdvertisingData::COMPLETE_LOCAL_NAME, (uint8_t *)DEVICE_NAME, sizeof(DEVICE_NAME));
    ble.setAdvertisingType(GapAdvertisingParams::ADV_CONNECTABLE_UNDIRECTED);
    ble.setAdvertisingInterval(1600); /* 1000ms; in multiples of 0.625ms. */
    ble.startAdvertising();

    uint8_t hrmCounter = 100;
    HeartRateService hrService(ble, hrmCounter, HeartRateService::LOCATION_FINGER);
    BatteryService battery(ble);
    DeviceInformationService deviceInfo(ble, "ARM", "Model1", "SN1", "hw-rev1", "fw-rev1", "soft-rev1");

    while (true) 
    {
        bool updateNow = false;
        if (triggerSensorPolling) 
        {
            triggerSensorPolling = false;
            bleUpdateCount++;
            if (bleUpdateCount >= 200)
            {
                bleUpdateCount = 0;
                indicatorLed2 = !indicatorLed2;
                updateNow = true;
            }
        }
        if (updateNow)
        {
            /* Do blocking calls or whatever is necessary for sensor polling. */
            /* In our case, we simply update the dummy HRM measurement. */
            hrmCounter++;
            if (hrmCounter == 175) 
            {
                hrmCounter = 100;
            }

            hrService.updateHeartRate(hrmCounter);
        } 
        else 
        {
            ble.waitForEvent();
        }
    }
}
